//
//	DIB Section
//
//		Copyright (c) 2000-2002 Chihiro.SAKAMOTO (HyperWorks)
//
#include "StdAfx.h"
#include "DrawImage.h"
#include "dc.h"
#include "Misc.h"

//
// RXgN^
//
CDrawImage::CDrawImage()
	:CImage(), hBitmap(0)
{
}

//
// fXgN^
//
CDrawImage::~CDrawImage()
{
	GdiFlush();
	if (hBitmap) {
		::DeleteObject(hBitmap);
	}
}

//
// DIBZNV̍쐬
//
BOOL CDrawImage::Create(HDC dc, int width, int height)
{
	W = width;
	H = height;
	D = 24;

	bytes_per_line = ScanBytes(width, 24);
	bytes_per_pixel = PixelBytes(24);

	InfoHeader.biSize			= sizeof(BITMAPINFOHEADER);
	InfoHeader.biWidth			= width;
	InfoHeader.biHeight			= height;
	InfoHeader.biBitCount		= 24;
	InfoHeader.biPlanes			= 1;
	InfoHeader.biXPelsPerMeter	= 0;
	InfoHeader.biYPelsPerMeter	= 0;
	InfoHeader.biClrUsed		= 0;
	InfoHeader.biClrImportant	= 0;
	InfoHeader.biCompression	= BI_RGB;
	InfoHeader.biSizeImage		= bytes_per_line * height;

	Info = (BITMAPINFO *)&InfoHeader;
	hBitmap = CreateDIBSection(dc, Info, DIB_RGB_COLORS, &Bits, NULL, 0);

	return hBitmap != 0;
}

//
// DIBɕ`悷
//
void CDrawImage::DrawText(HFONT hFont, int x1, int y1, const char *str, COLORREF color)
{
	CMemoryDC	dc(0);
	HBITMAP	oldBitmap = dc.SelectObject(hBitmap);
	HFONT	oldFont = dc.SelectObject(hFont);

	dc.SetTextColor(color);
	dc.SetBkMode(TRANSPARENT);

	dc.ExtTextOut(x1, y1, 0, 0, str, strlen(str), NULL);

	dc.SelectObject(oldBitmap);
	dc.SelectObject(oldFont);

	GdiFlush();
}

//
// CvC1Xebv
//
void CDrawImage::WipeIn(CImage *image, const CRect &rect, int count)
{
	int		len = rect.Width() * 3;
	for (int y=rect.top + count; y<rect.bottom; y+=8) {
		memcpy(GetBits(rect.left, y), image->GetBits(rect.left, y), len);
	}
}

//
// CvAEg1Xebv
//
void CDrawImage::WipeOut(const CRect &rect, int count)
{
	int		len = rect.Width() * 3;
	for (int y=rect.top + count; y<rect.bottom; y+=8) {
		memset(GetBits(rect.left, y), 0, len);
	}
}

//
// CvC21Xebv
//
bool CDrawImage::WipeIn2(CImage *image, const CRect &rect, int count)
{
	int width = rect.Width();
	int height = rect.Height();

	bool update = false;
	int npos = count * 4;
	for (int y = 0; y < height; y += 32) {
		if (npos >= 0 && npos < 32) {
			int ypos = y + npos;
			Copy(image, CRect(0, ypos, width, ypos + 4));
			update = true;
		}
		npos -= 4;
	}
	return update;
}

//
// CvAEg21Xebv
//
bool CDrawImage::WipeOut2(const CRect &rect, int count)
{
	int width = rect.Width();
	int height = rect.Height();

	bool update = false;
	int npos = count * 4;
	for (int y = 0; y < height; y += 32) {
		if (npos >= 0 && npos < 32) {
			int ypos = y + npos;
			FillRect(CRect(0, ypos, width, ypos + 4), RGB(0, 0, 0));
			update = true;
		}
		npos -= 4;
	}
	return update;
}

//
// tF[h1Xebv
//
void CDrawImage::FadeCvt(CImage *image, const CRect &rect, const byte_t *cvt)
{
	for (int y=rect.top; y<rect.bottom; y++) {
		byte_t *p1 = (byte_t *)GetBits(rect.left, y);
		byte_t *p2 = (byte_t *)image->GetBits(rect.left, y);

		for (int x=rect.left; x<rect.right; x++) {
			*p1++ = cvt[*p2++];		// blue
			*p1++ = cvt[*p2++];		// green
			*p1++ = cvt[*p2++];		// red
		}
	}
}

//
// u->摜vtF[h1Xebv
//
void CDrawImage::FadeFromBlack(CImage *image, const CRect &rect, int count)
{
	byte_t	cvt[256];

	count++;
	for (int i=0; i<256; i++) {
		cvt[i] = (byte_t)((i * count) / 16);
	}
	FadeCvt(image, rect, cvt);
}

//
// u摜->vtF[h1Xebv
//
void CDrawImage::FadeToBlack(CImage *image, const CRect &rect, int count)
{
	byte_t	cvt[256];

	count = 15 - count;
	for (int i=0; i<256; i++) {
		cvt[i] = (byte_t)((i * count) / 16);
	}
	FadeCvt(image, rect, cvt);
}

//
// u->摜vtF[h1Xebv
//
void CDrawImage::FadeFromWhite(CImage *image, const CRect &rect, int count)
{
	byte_t	cvt[256];

	count++;
	int		level = 255 * (16 - count);
	for (int i=0; i<256; i++) {
		cvt[i] = (byte_t)((i * count + level) / 16);
	}
	FadeCvt(image, rect, cvt);
}

//
// u摜->vtF[h1Xebv
//
void CDrawImage::FadeToWhite(CImage *image, const CRect &rect, int count)
{
	byte_t	cvt[256];

	count = 15 - count;
	int		level = 255 * (16 - count);
	for (int i=0; i<256; i++) {
		cvt[i] = (byte_t)((i * count + level) / 16);
	}
	FadeCvt(image, rect, cvt);
}

//
// ^C~LVO1Xebv
//
void CDrawImage::Mix(CImage *image, const CRect &rect, int count)
{
	static unsigned	BitMask[] = {
		0x2080,	// 0010 0000 1000 0000
		0xa0a0,	// 1010 0000 1010 0000
		0xa1a4,	// 1010 0001 1010 0100
		0xa5a5,	// 1010 0101 1010 0101
		0xada7,	// 1010 1101 1010 0111
		0xafaf,	// 1010 1111 1010 1111
		0xefbf,	// 1110 1111 1011 1111
		0xffff,	// 1111 1111 1111 1111
	} ;
	static unsigned	XMask[] = {
		0xf000, 0x0f00, 0x00f0, 0x000f,
	} ;
	static unsigned	YMask[] = {
		0x8888, 0x4444, 0x2222, 0x1111,
	} ;

	for (int y=rect.top; y<rect.bottom; y++) {
		unsigned char *p = (unsigned char *)GetBits(rect.left, y);
		byte_t *q = (byte_t *)image->GetBits(rect.left, y);
		unsigned	mask = BitMask[count] & YMask[y & 3];

		for (int x=rect.left; x<rect.right; x++) {
			if (mask & XMask[x & 3]) {
				p[0] = q[0];	// blue
				p[1] = q[1];	// green
				p[2] = q[2];	// red
			}
			p += 3;
			q += 3;
		}
	}
}
