//
//	XNvgRo[^
//
//		Copyright (c) 2000-2002 Chihiro.SAKAMOTO (HyperWorks)
//
#ifndef	__MAKESCRIPT_H__
#define	__MAKESCRIPT_H__

#include "Lexer.h"
#include "ScriptTypes.h"

#ifndef	_MAX_PATH
#define	_MAX_PATH	260
#endif

#define	UC(c)	(unsigned char)(c)

class FileReader;

//
//	XNvgRo[^NX
//
class MakeScript {
  protected:
	// G[bZ[W̎
	enum	{
		MsgNotice,
		MsgError,
		MsgFatal,
	} ;

  protected:
	//
	// xQƃNX
	//
	class LabelRef {
	  public:
		LabelRef(LabelRef *_next, long *_label_ref)
			:next(_next), label_ref(_label_ref) {}

	  public:
		LabelRef   *next;
		long       *label_ref;
	} ;

	//
	// xNX
	//
	class Label {
	  public:
		Label(const char *_label, unsigned _line, long _jmp_addr, LabelRef *_ref)
			:label(_label), line(_line), jmp_addr(_jmp_addr), ref(_ref) {}

	  public:
		string		label;
		unsigned	line;
		long		jmp_addr;
		LabelRef   *ref;
	} ;

	//
	// ϐ/萔NX
	//
	struct	ValueOrNumber	{
		bool	isvalue;
		int		value;
	} ;

	typedef void (MakeScript::*cmd_t)(Lexer &);
	typedef pair<const char *, cmd_t>	CmdTab;

	//
	// R}hrNX
	//
	class cmdless: public binary_function<const char *, const char *, bool> {
	  public:
		result_type operator()(first_argument_type x, second_argument_type y) const
		{
			return stricmp(x, y) < 0;
		}
	} ;
	typedef map<const char *, cmd_t, cmdless>	cmdmap;

  public:
	MakeScript();
	~MakeScript();

	int ReadScript(const char *name);
	void ParserString(const char *str);
	int WriteScript(const char *name);

	bool IsError() const { return nerror != 0; }

  protected:
	char *ThenLabel();
	void AddLabel(const char *label);
	void FindLabel(const char *label, long *reference);
	void LabelCheck();
	void SetLabel(Lexer &lexer);
	void *AllocCommand(int size, int cmd);
	bool ChkKeyword(const char *str, const char *keyword);
	int GetPosition(const char *str);
	int GetUpdateType(const char *str);
	bool GetValueOrNumber(ValueOrNumber *value, Lexer &lexer);
	int BoolOp(const char *op);
	int NegBoolOp(const char *op);
	void CommandOnlyCmd(Lexer &lexer, int cmd);
	void SetCmd(Lexer &lexer);
	void GotoCmd(Lexer &lexer);
	void IfCmd(Lexer &lexer);
	void ElseCmd(Lexer &lexer);
	void EndifCmd(Lexer &lexer);
	void MenuCmd(Lexer &lexer);
	void ExecCmd(Lexer &lexer);
	void LoadCmd(Lexer &lexer);
	void UpdateCmd(Lexer &lexer);
	void ClearCmd(Lexer &lexer);
	void WaitCmd(Lexer &lexer);
	void MusicCmd(Lexer &lexer);
	void SoundCmd(Lexer &lexer);
	bool ChkTermination(const char *str);
	void TextCmd(Lexer &lexer);
	void ModeCmd(Lexer &lexer);
	void SystemCmd(Lexer &lexer);
	void StopmCmd(Lexer &lexer);
	void FadeInCmd(Lexer &lexer);
	void FadeOutCmd(Lexer &lexer);
	void WipeInCmd(Lexer &lexer);
	void WipeOutCmd(Lexer &lexer);
	void CutInCmd(Lexer &lexer);
	void CutOutCmd(Lexer &lexer);
	void WhiteInCmd(Lexer &lexer);
	void WhiteOutCmd(Lexer &lexer);
	void FlashCmd(Lexer &lexer);
	void ShakeCmd(Lexer &lexer);
	void EndCmd(Lexer &lexer);

	cmd_t ParseCommand(Lexer &lexer);

	bool OpenValueTable();
	bool CloseValueTable();
	int FindValue(const char *value);

	unsigned AddMessage(const char *msg, int limit);

	virtual void OutputMessage(int code, const char *str) const;
	virtual int GetErrorPrefix(char *str) const;
	virtual const char *GetValueFile() const;

	void __cdecl ErrorMessage(const char *fmt, ...);
	void __cdecl Error(const char *fmt, ...);
	void __cdecl FatalError(const char *fmt, ...);
	void __cdecl Notice(const char *fmt, ...);

  protected:
	FileReader     *reader;

  private:
	int				nerror;
	unsigned		then_index;
	stack<unsigned>	then_nest;
	int				ncommand;
	bool			add_value;
	vector<string>	value_name;
	vector<Label>	labels;
	char           *command_buffer;
	cmdmap			cmd_table;
} ;

#endif
